@@ -1,13 +1,24 @@
-# Copyright 2008-2012 Gentoo Foundation
+# Copyright 2008-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
-from _emerge.SubProcess import SubProcess
+try:
+	import fcntl
+except ImportError:
+	# http://bugs.jython.org/issue1074
+	fcntl = None
+
+import platform
 import sys
+
+from _emerge.SubProcess import SubProcess
 import portage
 from portage import os
 from portage.const import BASH_BINARY
 from portage.util._async.PipeLogger import PipeLogger
 
+# On Darwin, FD_CLOEXEC triggers errno 35 for stdout (bug #456296)
+_disable_cloexec_stdout = platform.system() in ("Darwin",)
+
 class SpawnProcess(SubProcess):
 
 	"""
@@ -18,7 +29,7 @@ class SpawnProcess(SubProcess):
 
 	_spawn_kwarg_names = ("env", "opt_name", "fd_pipes",
 		"uid", "gid", "groups", "umask", "logfile",
-		"path_lookup", "pre_exec")
+		"path_lookup", "pre_exec", "close_fds")
 
 	__slots__ = ("args",) + \
 		_spawn_kwarg_names + ("_pipe_logger", "_selinux_type",)
@@ -64,17 +75,20 @@ class SpawnProcess(SubProcess):
 
 		fd_pipes_orig = fd_pipes.copy()
 
-		if log_file_path is not None:
+		if log_file_path is not None or self.background:
 			fd_pipes[1] = slave_fd
 			fd_pipes[2] = slave_fd
 
 		else:
-			# Create a dummy pipe so the scheduler can monitor
-			# the process from inside a poll() loop.
-			fd_pipes[self._dummy_pipe_fd] = slave_fd
-			if self.background:
-				fd_pipes[1] = slave_fd
-				fd_pipes[2] = slave_fd
+			# Create a dummy pipe that PipeLogger uses to efficiently
+			# monitor for process exit by listening for the EOF event.
+			# Re-use of the allocated fd number for the key in fd_pipes
+			# guarantees that the keys will not collide for similarly
+			# allocated pipes which are used by callers such as
+			# FileDigester and MergeProcess. See the _setup_pipes
+			# docstring for more benefits of this allocation approach.
+			self._dummy_pipe_fd = slave_fd
+			fd_pipes[slave_fd] = slave_fd
 
 		kwargs = {}
 		for k in self._spawn_kwarg_names:
@@ -96,7 +110,7 @@ class SpawnProcess(SubProcess):
 			# spawn failed
 			self._unregister()
 			self._set_returncode((self.pid, retval))
-			self.wait()
+			self._async_wait()
 			return
 
 		self.pid = retval[0]
@@ -105,6 +119,20 @@ class SpawnProcess(SubProcess):
 		stdout_fd = None
 		if can_log and not self.background:
 			stdout_fd = os.dup(fd_pipes_orig[1])
+			if fcntl is not None and not _disable_cloexec_stdout:
+				try:
+					fcntl.FD_CLOEXEC
+				except AttributeError:
+					pass
+				else:
+					try:
+						fcntl.fcntl(stdout_fd, fcntl.F_SETFL,
+							fcntl.fcntl(stdout_fd,
+							fcntl.F_GETFL) | fcntl.FD_CLOEXEC)
+					except IOError:
+						# FreeBSD may return "Inappropriate ioctl for device"
+						# error here (ENOTTY).
+						pass
 
 		self._pipe_logger = PipeLogger(background=self.background,
 			scheduler=self.scheduler, input_fd=master_fd,
