@@ -1,8 +1,9 @@
-# Copyright 2010-2012 Gentoo Foundation
+# Copyright 2010-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 import tempfile
 
+import portage
 from portage import os
 from portage import shutil
 from portage.const import EBUILD_PHASES
@@ -12,6 +13,7 @@ from portage.package.ebuild.doebuild import doebuild_environment
 from portage.package.ebuild.prepare_build_dirs import prepare_build_dirs
 from portage.util._async.SchedulerInterface import SchedulerInterface
 from portage.util._eventloop.EventLoop import EventLoop
+from portage.util._eventloop.global_event_loop import global_event_loop
 from _emerge.EbuildPhase import EbuildPhase
 
 def spawn_nofetch(portdb, ebuild_path, settings=None):
@@ -79,7 +81,8 @@ def spawn_nofetch(portdb, ebuild_path, settings=None):
 		prepare_build_dirs(settings=settings)
 		ebuild_phase = EbuildPhase(background=False,
 			phase='nofetch',
-			scheduler=SchedulerInterface(EventLoop(main=False)),
+			scheduler=SchedulerInterface(portage._internal_caller and
+				global_event_loop() or EventLoop(main=False)),
 			settings=settings)
 		ebuild_phase.start()
 		ebuild_phase.wait()
