@@ -1,4 +1,4 @@
-# Copyright 2011-2012 Gentoo Foundation
+# Copyright 2011-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 import subprocess
@@ -7,7 +7,8 @@ import sys
 import portage
 from portage import os
 from portage import _unicode_decode
-from portage.const import PORTAGE_BIN_PATH, PORTAGE_PYM_PATH, USER_CONFIG_PATH
+from portage.const import (BASH_BINARY, PORTAGE_BASE_PATH,
+	PORTAGE_BIN_PATH, PORTAGE_PYM_PATH, USER_CONFIG_PATH)
 from portage.process import find_binary
 from portage.tests import TestCase
 from portage.tests.resolver.ResolverPlayground import ResolverPlayground
@@ -165,6 +166,8 @@ pkg_preinst() {
 		cachedir_pregen = os.path.join(portdir, "metadata", "md5-cache")
 
 		portage_python = portage._python_interpreter
+		dispatch_conf_cmd = (portage_python, "-Wd",
+			os.path.join(PORTAGE_BIN_PATH, "dispatch-conf"))
 		ebuild_cmd = (portage_python, "-Wd",
 			os.path.join(PORTAGE_BIN_PATH, "ebuild"))
 		egencache_cmd = (portage_python, "-Wd",
@@ -175,6 +178,8 @@ pkg_preinst() {
 			os.path.join(PORTAGE_BIN_PATH, "emaint"))
 		env_update_cmd = (portage_python, "-Wd",
 			os.path.join(PORTAGE_BIN_PATH, "env-update"))
+		etc_update_cmd = (BASH_BINARY,
+			os.path.join(PORTAGE_BIN_PATH, "etc-update"))
 		fixpackages_cmd = (portage_python, "-Wd",
 			os.path.join(PORTAGE_BIN_PATH, "fixpackages"))
 		portageq_cmd = (portage_python, "-Wd",
@@ -198,6 +203,10 @@ pkg_preinst() {
 
 		test_commands = (
 			env_update_cmd,
+			portageq_cmd + ("envvar", "-v", "CONFIG_PROTECT", "EROOT",
+				"PORTAGE_CONFIGROOT", "PORTAGE_TMPDIR", "USERLAND"),
+			etc_update_cmd,
+			dispatch_conf_cmd,
 			emerge_cmd + ("--version",),
 			emerge_cmd + ("--info",),
 			emerge_cmd + ("--info", "--verbose"),
@@ -302,6 +311,7 @@ pkg_preinst() {
 			"PORTAGE_PYTHON" : portage_python,
 			"PORTAGE_TMPDIR" : portage_tmpdir,
 			"PYTHONPATH" : pythonpath,
+			"__PORTAGE_TEST_PATH_OVERRIDE" : fake_bin,
 		}
 
 		if "__PORTAGE_TEST_HARDLINK_LOCKS" in os.environ:
@@ -312,7 +322,11 @@ pkg_preinst() {
 		dirs = [cachedir, cachedir_pregen, distdir, fake_bin,
 			portage_tmpdir, updates_dir,
 			user_config_dir, var_cache_edb]
-		true_symlinks = ["chown", "chgrp"]
+		etc_symlinks = ("dispatch-conf.conf", "etc-update.conf")
+		# Override things that may be unavailable, or may have portability
+		# issues when running tests in exotic environments.
+		#   prepstrip - bug #447810 (bash read builtin EINTR problem)
+		true_symlinks = ["find", "prepstrip", "sed", "scanelf"]
 		true_binary = find_binary("true")
 		self.assertEqual(true_binary is None, False,
 			"true command not found")
@@ -321,6 +335,9 @@ pkg_preinst() {
 				ensure_dirs(d)
 			for x in true_symlinks:
 				os.symlink(true_binary, os.path.join(fake_bin, x))
+			for x in etc_symlinks:
+				os.symlink(os.path.join(PORTAGE_BASE_PATH, "cnf", x),
+					os.path.join(eprefix, "etc", x))
 			with open(os.path.join(var_cache_edb, "counter"), 'wb') as f:
 				f.write(b"100")
 			# non-empty system set keeps --depclean quiet
